@@ -1,3 +1,4 @@
+# noinspection PyUnresolvedReferences
 """Interpretable Federated Rule Algorithm
 
 **Federated Learning** allows several sources (*nodes*) of data to contribute to a single model without sharing their
@@ -85,7 +86,7 @@ Example of step 3: you could create a central configuration json file contaning:
 Example of step 4:
 >>> from ifra import Node
 >>> public_config_path = "tests/real/node_0/public_configs.json"
->>> data_config_path = "tests/real/node_0/path_configs.json"
+>>> data_config_path = "tests/real/node_0/data_configs.json"
 >>> thenode = Node(path_public_configs=public_config_path, path_data=data_config_path)
 >>> thenode.watch()
 
@@ -108,6 +109,7 @@ from .datapreps import DataPrep
 from .updaters import Updater
 from .fitters import Fitter
 from .aggregations import Aggregation
+from .setup_logger import setup_logger
 
 try:
     from ._version import __version__
