@@ -1,6 +1,8 @@
-from copy import copy
+from copy import copy, deepcopy
 from pathlib import Path
-from typing import Union
+from time import sleep, time
+from typing import Union, Optional
+from json import load
 
 from transparentpath import TransparentPath
 
@@ -40,39 +42,70 @@ class Config:
     """Configuration keys that CAN figure in the json file"""
 
     # noinspection PyUnresolvedReferences
-    def __init__(self, path: Union[str, Path, TransparentPath]):
+    def __init__(self, path: Optional[Union[str, Path, TransparentPath]] = None, configs: Optional[dict] = None):
         """
         Parameters
         ----------
-        path: Union[str, Path, TransparentPath]
+        path: Optional[Union[str, Path, TransparentPath]]
             The path to the json file containing the configuration. Will be casted into a transparentpath if a str is
             given, so make sure that if you pass a str that should be local, you did not set a global file system.
+        configs: Optional[dict]
+            If path is not specified, you can give the dictionnary of configuration directly. No checks are done on the
+            keys then.
         """
+        self.path = None
+        self.configs = None
+        if path is not None and configs is not None:
+            raise ValueError("Only and only one of 'path' and 'configs' must be specified")
+        if path is None and configs is None:
+            raise ValueError("Only and only one of 'path' and 'configs' must be specified")
+        if path is not None:
+            self._init_with_path(path)
+        else:
+            self._init_with_configs(configs)
 
+    def _init_with_path(self, path: Union[str, Path, TransparentPath]):
         if type(path) == str:
             path = TransparentPath(path)
 
         if not path.suffix == ".json":
             raise ValueError("Config path must be a json")
 
-        if hasattr(path, "read"):
-            self.configs = path.read()
-        else:
-            with open(path) as opath:
-                self.configs = load(opath)
+        lockfile = path.append(".locked")
+        t = time()
+        while lockfile.isfile():
+            sleep(0.5)
+            if time() - t > 5:
+                raise FileExistsError(f"File {lockfile} exists : could not lock on file {path}")
+        lockfile.touch()
 
-        self.path = path
+        try:
+            if hasattr(path, "read"):
+                self.configs = path.read()
+            else:
+                with open(path) as opath:
+                    self.configs = load(opath)
 
-        for key in self.EXPECTED_CONFIGS:
-            if key not in self.configs:
-                raise IndexError(f"Missing required config {key}")
-        for key in self.configs:
-            if key not in self.EXPECTED_CONFIGS and key not in self.ADDITIONNAL_CONFIGS:
-                raise IndexError(f"Unexpected config {key}")
+            self.path = path
 
-        for key in self.configs:
-            if self.configs[key] == "":
-                self.configs[key] = None
+            for key in self.EXPECTED_CONFIGS:
+                if key not in self.configs:
+                    raise IndexError(f"Missing required config {key}")
+            for key in self.configs:
+                if key not in self.EXPECTED_CONFIGS and key not in self.ADDITIONNAL_CONFIGS:
+                    raise IndexError(f"Unexpected config {key}")
+
+            for key in self.configs:
+                if self.configs[key] == "":
+                    self.configs[key] = None
+
+            lockfile.rm(absent="ignore")
+        except Exception as e:
+            lockfile.rm(absent="ignore")
+            raise e
+
+    def _init_with_configs(self, configs: dict):
+        self.configs = configs
 
     def __getattr__(self, item):
         if item == "configs":
@@ -91,19 +124,39 @@ class Config:
             raise ValueError(f"Configuration named '{item}' is not allowed")
         self.configs[item] = value
 
+    def __deepcopy__(self, memo):
+        cls = self.__class__
+        result = cls.__new__(cls)
+        memo[id(self)] = result
+        setattr(result, "configs", deepcopy(self.configs, memo))
+        return result
+
     def save(self) -> None:
         """Saves the current configuration into the file it used to load. This allows the user to change the
         configuration in code and save it. Note that one can not have added a key not present in
         `Config.EXPECTED_CONFIGS`"""
         to_save = copy(self.configs)
 
-        for key in to_save:
-            if key not in self.EXPECTED_CONFIGS and key not in self.ADDITIONNAL_CONFIGS:
-                raise ValueError(f"Forbidden configuration '{key}'")
-            if to_save[key] is None:
-                to_save[key] = ""
+        lockfile = self.path.append(".locked")
+        t = time()
+        while lockfile.isfile():
+            sleep(0.5)
+            if time() - t > 5:
+                raise FileExistsError(f"File {lockfile} exists : could not lock on file {self.path}")
+        lockfile.touch()
 
-        self.path.write(to_save)
+        try:
+            for key in to_save:
+                if key not in self.EXPECTED_CONFIGS and key not in self.ADDITIONNAL_CONFIGS:
+                    raise ValueError(f"Forbidden configuration '{key}'")
+                if to_save[key] is None:
+                    to_save[key] = ""
+
+            self.path.write(to_save)
+            lockfile.rm(absent="ignore")
+        except Exception as e:
+            lockfile.rm(absent="ignore")
+            raise e
 
 
 class NodePublicConfig(Config):
@@ -175,6 +228,10 @@ class NodePublicConfig(Config):
           * adaboost_updater (see `ifra.updaters.AdaboostUpdater`)\n
     stop: bool
         Set to True by `ifra.central_server.CentralServer` when the learning is over.
+    error: str
+        Will contain any error that `ifra.node.Node.watch` have raised
+    central_error: str
+        Will contain any error that `ifra.central_server.CentralServer.watch` have raised
     """
 
     EXPECTED_CONFIGS = [
@@ -195,7 +252,7 @@ class NodePublicConfig(Config):
         "fitter",
         "updater"
     ]
-    ADDITIONNAL_CONFIGS = ["stop"]
+    ADDITIONNAL_CONFIGS = ["stop", "error", "central_error"]
 
     def __init__(self, path: Union[str, TransparentPath]):
         super().__init__(path)
@@ -205,14 +262,19 @@ class NodePublicConfig(Config):
             self.configs["central_model_path"] = TransparentPath(self.configs["central_model_path"])
         if "stop" not in self.configs:
             self.configs["stop"] = False
+        if "error" not in self.configs:
+            self.configs["error"] = None
+        if "central_error" not in self.configs:
+            self.configs["central_error"] = None
 
     def __eq__(self, other):
         if not isinstance(other, NodePublicConfig):
             return False
         for key in NodePublicConfig.EXPECTED_CONFIGS:
-            if key == "local_model_path" or key == "central_model_path":  # Those parameters can change without problem
+            # Those parameters can change without problem
+            if key == "local_model_path" or key == "central_model_path" or key == "id":
                 continue
-            if self[key] != other[key]:
+            if self.configs[key] != other.configs[key]:
                 return False
         return True
 
@@ -274,7 +336,7 @@ class NodeDataConfig(Config):
     a dataframe (a single column in the case of the target file).
     The features file should contain one column for EACH FEATURE USED IN THE LEARNING, even if not all of them have data
     in this node. The order of the columns should match the order in
-    `ifra.central_server.NodePublicConfig`'s features_names`.
+    `ifra.central_server.NodeGate.NodePublicConfig`'s features_names`.
 
     Attributes
     ----------
