@@ -1,3 +1,4 @@
+import traceback
 from datetime import datetime
 from pathlib import Path
 from time import time, sleep
@@ -5,6 +6,7 @@ from time import time, sleep
 from typing import Union, Optional
 
 from ruleskit import RuleSet
+from tablewriter import TableWriter
 from transparentpath import TransparentPath
 import logging
 
@@ -98,7 +100,7 @@ class Node:
 
         if not len(list(self.public_configs.local_model_path.parent.ls(""))) == 0:
             raise ValueError(
-                "Path were Node model should be saved is not empty. Make sure you cleaned all previous learning output"
+                "Path where Node model should be saved is not empty. Make sure you cleaned all previous learning output"
             )
 
         self.data = NodeDataConfig(path_data)
@@ -110,7 +112,6 @@ class Node:
         else:
             self.dataprep = None
 
-        self.dataprep = None
         self.datapreped = False
         self.copied = False
         self.ruleset = None
@@ -136,12 +137,16 @@ class Node:
             `node.Node` *ruleset*
         """
         if self.public_configs.plot_data:
+            if not (self.data.x.parent / "plots").isdir():
+                (self.data.x.parent / "plots").mkdir()
             self.plot_data_histogram(self.data.x.parent / "plots")
 
         if self.dataprep is not None and not self.datapreped:
             logger.info(f"Datapreping data of node {self.public_configs.id}...")
             self.dataprep.dataprep()
             if self.public_configs.plot_data:
+                if not (self.data.x.parent / "plots_datapreped").isdir():
+                    (self.data.x.parent / "plots_datapreped").mkdir()
                 self.plot_data_histogram(self.data.x.parent / "plots_datapreped")
             self.datapreped = True
             logger.info(f"...datapreping done for node {self.public_configs.id}")
@@ -158,6 +163,7 @@ class Node:
 
         logger.info(f"Fitting node {self.public_configs.id} using {self.public_configs.fitter}...")
         self.ruleset = self.fitter.fit()
+        logger.info(f"Found {len(self.ruleset)} rules in node {self.public_configs.id}")
         self.ruleset_to_file()
         logger.info(f"... node {self.public_configs.id} fitted, results saved in"
                     f" {self.public_configs.local_model_path.parent}.")
@@ -174,12 +180,13 @@ class Node:
         """
         logger.info(f"Updating node {self.public_configs.id}...")
         # Compute activation of the selected rules
-        self.updater(ruleset)
+        self.updater.update(ruleset)
         logger.info(f"... node {self.public_configs.id} updated.")
 
     def ruleset_to_file(self) -> None:
         """Saves `ifra.node.Node` *ruleset* to `ifra.configs.NodePublicConfig` *local_model_path*,
         overwritting any existing file here, and in another file in the same directory but with a unique name.
+        Will also produce a .pdf of the ruleset using TableWriter.
         Does not do anything if `ifra.node.Node` *ruleset* is None
         """
 
@@ -196,6 +203,18 @@ class Node:
         ruleset.save(path)
         ruleset.save(self.public_configs.local_model_path)
 
+        try:
+            path_table = path.with_suffix(".pdf")
+            TableWriter(
+                path_table,
+                path.read(index_col=0).apply(
+                    lambda x: x.round(3) if x.dtype == float else x
+                ),
+                paperwidth=30
+            ).compile(clean_tex=True)
+        except ValueError:
+            logger.warning("Failed to produce tablewriter. Is LaTeX installed ?")
+
     def plot_data_histogram(self, path: TransparentPath) -> None:
         """Plots the distribution of the data located in `ifra.node.Node`'s *data.x* and
         `ifra.node.Node`'s *data.y* and save them in unique files.
@@ -203,8 +222,7 @@ class Node:
         Parameters
         ----------
         path: TransparentPath
-            Path to save the data. Should be a file. Will not be overwritten, a new file name will be created if it
-            already exists.
+            Path to save the data. Should be a directory.
         """
         x = self.data.x.read(**self.data.x_read_kwargs)
         for col, name in zip(x.columns, self.public_configs.features_names):
@@ -216,7 +234,7 @@ class Node:
 
             iteration = 0
             name = name.replace(' ', '_')
-            path_x = path.parent / f"{name}_{iteration}.pdf"
+            path_x = path / f"{name}_{iteration}.pdf"
             while path_x.isfile():
                 iteration += 1
                 path_x = path_x.parent / f"{name}_{iteration}.pdf"
@@ -230,14 +248,14 @@ class Node:
         )
 
         iteration = 0
-        path_y = path.parent / f"classes_{iteration}.pdf"
+        path_y = path / f"classes_{iteration}.pdf"
         while path_y.isfile():
             iteration += 1
             path_y = path_y.parent / f"classes_{iteration}.pdf"
 
         fig.savefig(path_y)
 
-    def watch(self, timeout:int = 0, sleeptime: int = 5) -> None:
+    def watch(self, timeout: int = 0, sleeptime: int = 1) -> None:
         """Monitors new changes in the central server, every *sleeptime* seconds for *timeout* seconds, triggering
         node fit when a new model is found, or if the function just started. Sets
        ` ifra.configs.NodePublicConfig` *id* by re-reading the configuration file if it is None.
@@ -253,47 +271,69 @@ class Node:
             How many seconds between each checks for new central model. Default value = 5
         """
 
-        def get_ruleset() -> None:
-            """Fetch the central server's latest model's RuleSet.
-            `ifra.node.Node.last_fetch` will be set to now and `ifra.node.Node.new_data` to True."""
-            central_ruleset = RuleSet()
-            central_ruleset.load(self.public_configs.central_model_path)
-            self.last_fetch = datetime.now()
-            self.update_from_central(central_ruleset)
-            logger.info(f"Fetched new central ruleset in node {self.public_configs.id} at {self.last_fetch}")
-            self.new_data = True
+        try:
+            def get_ruleset() -> None:
+                """Fetch the central server's latest model's RuleSet.
+                `ifra.node.Node.last_fetch` will be set to now and `ifra.node.Node.new_data` to True."""
+                central_ruleset = RuleSet()
+                central_ruleset.load(self.public_configs.central_model_path)
+                self.last_fetch = datetime.now()
+                self.update_from_central(central_ruleset)
+                logger.info(f"Fetched new central ruleset in node {self.public_configs.id} at {self.last_fetch}")
+                self.new_data = True
 
-        t = time()
-        new_central_model = True  # start at true to trigger fit even if no central model is here at first iteration
-        while time() - t < timeout or timeout <= 0:
-            self.public_configs = NodePublicConfig(self.path_public_configs)
-            if self.public_configs.stop:
-                logger.info(f"Stopping learning in node {self.public_configs.id}")
-                del self.public_configs.stop
-                self.public_configs.save()
-                break
-            if self.public_configs.id is None:
-                logger.warning(
-                    "Node id is not set, meaning central server is not running. Waiting for central server to start."
-                )
-                continue
+            t = time()
+            started = False
+            stop_message = True
+            new_central_model = True  # start at true to trigger fit even if no central model is here at first iteration
+            while time() - t < timeout or timeout <= 0:
+                if not (self.path_public_configs.parent / "running").isfile():
+                    (self.path_public_configs.parent / "running").touch()
+                self.public_configs = NodePublicConfig(self.path_public_configs)
+                if self.public_configs.id is None:
+                    raise ValueError(
+                        "Node id is not set, meaning central server is not running. Always start central server first."
+                    )
+                if started is False:
+                    logger.info(f"Starting node {self.public_configs.id}")
+                    started = True
+                if self.public_configs.central_error is not None:
+                    logger.info(f"Central server crashed. Stopping node {self.public_configs.id}.")
+                    stop_message = False
+                    del self.public_configs.configs["central_error"]
+                    self.public_configs.save()
+                    break
+                if self.public_configs.stop:
+                    logger.info(f"'stop' flag is True. Stopping learning in node {self.public_configs.id}")
+                    stop_message = False
+                    del self.public_configs.configs["stop"]
+                    self.public_configs.save()
+                    break
 
-            if self.last_fetch is None:
-                if self.public_configs.central_model_path.isfile():
-                    get_ruleset()
-                    new_central_model = True
-            else:
-                if (
-                    self.public_configs.central_model_path.isfile()
-                    and self.public_configs.central_model_path.info()["mtime"] > self.last_fetch.timestamp()
-                ):
-                    get_ruleset()
-                    new_central_model = True
+                if self.last_fetch is None:
+                    if self.public_configs.central_model_path.isfile():
+                        get_ruleset()
+                        new_central_model = True
+                else:
+                    if (
+                        self.public_configs.central_model_path.isfile()
+                        and self.public_configs.central_model_path.info()["mtime"] > self.last_fetch.timestamp()
+                    ):
+                        get_ruleset()
+                        new_central_model = True
 
-            if new_central_model:
-                self.fit()
-                new_central_model = False
-            sleep(sleeptime)
+                if new_central_model:
+                    self.fit()
+                    new_central_model = False
+                sleep(sleeptime)
 
-        if not self.public_configs.stop:
-            logger.info(f"Timeout of {timeout} seconds reached, stopping learning in node {self.public_configs.id}.")
+            if stop_message:
+                logger.info(f"Timeout of {timeout} seconds reached, stopping learning in node"
+                            f" {self.public_configs.id}.")
+
+            (self.path_public_configs.parent / "running").rm(absent="ignore")
+        except Exception as e:
+            (self.path_public_configs.parent / "running").rm(absent="ignore")
+            self.public_configs.error = traceback.format_exc()
+            self.public_configs.save()
+            raise e
